/* (C) 2022 YiRing, Inc. */
package com.yiring.app.design.strategy.rule;

import com.yiring.app.constant.alarm.RelevanceParamEnum;
import com.yiring.app.design.strategy.IParamInitStrategy;
import com.yiring.app.design.strategy.IScanAlarmStrategy;
import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.domain.location.*;
import com.yiring.app.param.location.fence.LocationFenceJobParam;
import com.yiring.app.param.location.rule.*;
import com.yiring.app.util.TimeUtil;
import com.yiring.app.vo.location.fence.LocationFenceJobVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

/**
 * 最多人数相关策略
 * @author tml
 * @version 1.0
 * @date 2022/4/29 9:28
 */
@Component
public class MaxPeopleNumberStrategy implements IParamInitStrategy<LocationFenceRuleAddParam>, IScanAlarmStrategy {

    @Resource
    private LocationTurnoverRepository locationTurnoverRepository;

    public MaxPeopleNumberStrategy() {
        Integer relevanceParam = RelevanceParamEnum.MAX_PEOPLE_NUMBER.getCode();
        LocationFenceRuleContext.register(relevanceParam, this);
        ScanAlarmContext.register(relevanceParam, this);
    }

    @Override
    public Result<String> paramInit(LocationFenceRuleAddParam param) {
        RuleParam ruleParam = param.getRuleParam();
        List<TimeAndNumberParam> timeAndNumber = ruleParam.getTimeAndNumber();
        if (CollectionUtils.isEmpty(timeAndNumber)) {
            return Result.no(Status.EXPECTATION_FAILED, "请至少配置一条最多人数的规则");
        }
        for (TimeAndNumberParam item : timeAndNumber) {
            if (item.getNumber() == null || item.getNumber() < 0) {
                Result.no(Status.EXPECTATION_FAILED, "最大人数不能空或小于零");
            }
        }
        //如果只设置星期没有设置时间，则默认时间是全天
        for (TimeAndNumberParam item : timeAndNumber) {
            for (TimeParam time : item.getTime()) {
                if (time.getWeek() != null) {
                    if (time.getBeginTime() == null) {
                        time.setBeginTime("00:00:00");
                    }
                    if (time.getEndTime() == null) {
                        time.setEndTime("23:59:59");
                    }
                }
            }
        }
        //将时间和距离都设为-1
        ruleParam.setDuration(null);
        ruleParam.setDistance(null);
        ArrayList<TimeAndUserParam> list = new ArrayList<>();
        ruleParam.setTimeAndUser(list);
        return Result.ok();
    }

    @Override
    public LocationFenceJobVo scanAlarm(LocationFenceJobParam param) {
        RuleParam rule = param.getRule();
        Set<LocationTag> tagSet = param.getTagSet();
        Long mapId = param.getMapId();
        List<LocationFenceAlarm> oldAlarms = param.getOldAlarms();
        Long fenceId = param.getFenceId();
        Long alarmTypeId = param.getAlarmTypeId();
        LocationFenceJobVo jobVo = new LocationFenceJobVo();
        List<TimeAndNumberParam> timeAndNumberParamList = rule.getTimeAndNumber();
        TimeAndNumberParam timeAndNumberParam = null;
        StringBuilder begin = new StringBuilder();
        StringBuilder end = new StringBuilder();
        //筛选在当前时间生效的配置
        for (TimeAndNumberParam item : timeAndNumberParamList) {
            for (TimeParam time : item.getTime()) {
                if (TimeUtil.judgeWeek(LocalDateTime.now(), time.getWeek())) {
                    if (TimeUtil.judgeTime(LocalDateTime.now(), time.getBeginTime(), time.getEndTime())) {
                        begin.append(time.getWeek()).append(" ").append(time.getBeginTime());
                        end.append(time.getWeek()).append(" ").append(time.getEndTime());
                        timeAndNumberParam = item;
                        break;
                    }
                }
            }
        }
        //如果现在不在配置的时间内，或围栏中的人少于配置的最大人数，则将报警记录中的状态刷新为结束，并且需要添加的报警为空
        if (timeAndNumberParam == null || tagSet.size() <= timeAndNumberParam.getNumber()) {
            jobVo.setModifyAlarm(oldAlarms);
            return jobVo;
        }
        //多出的人数
        int exceed = tagSet.size() - timeAndNumberParam.getNumber();
        List<LocationFenceAlarm> newAlarms = new ArrayList<>();
        //查询围栏内人员进入时的信息
        List<LocationTurnover> turnoverList = locationTurnoverRepository.withinFence(fenceId, true);
        //筛选出在配置时间段内进入的进出信息
        turnoverList =
            turnoverList
                .stream()
                .filter(e -> TimeUtil.judgeWeekTime(e.getTime(), begin.toString(), end.toString()))
                .sorted(Comparator.comparing(LocationTurnover::getTime).reversed())
                .collect(Collectors.toList());
        //记录触发报警的信息(例如最大人数是3，那么前三个员工进入的不会记录)
        for (int i = 0; i < exceed; i++) {
            LocationTurnover turnover = turnoverList.get(i);
            LocationFence fence = LocationFence.builder().id(fenceId).name(param.getFenceName()).build();
            AlarmType alarmType = AlarmType.builder().id(alarmTypeId).build();
            LocationFenceAlarm locationFenceAlarm = LocationFenceAlarm
                .builder()
                .fence(fence)
                .point(turnover.getTag().getPoint())
                .areaId(mapId)
                .user(turnover.getTag().getUser())
                .tag(turnover.getTag())
                .startTime(LocalDateTime.now())
                .type(alarmType)
                .status(LocationFenceAlarm.Status.ING)
                .build();
            newAlarms.add(locationFenceAlarm);
        }
        //取两个集合没有相交的部分，newAlarms不重合的部分代表需要添加的报警，oldAlarms不重合的部分代表需要结束的报警
        IScanAlarmStrategy.notIntersect(newAlarms, oldAlarms, jobVo);
        return jobVo;
    }
}
