/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.key.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.key.KeyAlarmRule;
import com.yiring.app.domain.key.KeyAlarmRuleRepository;
import com.yiring.app.excel.key.KeyAlarmRuleExportExcel;
import com.yiring.app.param.key.KeyAlarmRuleAddParam;
import com.yiring.app.param.key.KeyAlarmRuleExportParam;
import com.yiring.app.param.key.KeyAlarmRuleModifyParam;
import com.yiring.app.param.key.KeyAlarmRulePageParam;
import com.yiring.app.service.key.KeyAlarmRuleService;
import com.yiring.app.vo.key.KeyAlarmRuleVo;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 按键报警规则
 *
 * @author LJ-2204
 * @date 2022/5/6
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class KeyAlarmRuleServiceImpl implements KeyAlarmRuleService {

    @Resource
    KeyAlarmRuleRepository keyAlarmRuleRepository;

    @Override
    public Result<String> add(KeyAlarmRuleAddParam keyAlarmRuleAddParam) {
        long count = keyAlarmRuleRepository.count(
            Example.of(
                KeyAlarmRule
                    .builder()
                    .department(Department.builder().id(keyAlarmRuleAddParam.getDeptId()).build())
                    .build()
            )
        );

        if (count > 0) {
            return Result.no(Status.BAD_REQUEST);
        }

        HashSet<User> users = new HashSet<>();
        String[] strings = keyAlarmRuleAddParam.getUserIds().split(",");
        Long[] ids = Convert.toLongArray(strings);
        if (ids.length < 3) {
            return Result.no(Status.BAD_REQUEST, "报警接收人至少3个");
        }
        for (Long id : ids) {
            users.add(User.builder().id(id).build());
        }
        KeyAlarmRule keyAlarmRule = KeyAlarmRule
            .builder()
            .department(Department.builder().id(keyAlarmRuleAddParam.getDeptId()).build())
            .users(users)
            .types(keyAlarmRuleAddParam.getTypes())
            .build();
        keyAlarmRuleRepository.save(keyAlarmRule);
        return Result.ok();
    }

    @Override
    public Result<String> delete(IdParam idParam) {
        Optional<KeyAlarmRule> keyAlarmRuleOptional = keyAlarmRuleRepository.findById(idParam.getId());

        if (keyAlarmRuleOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "报警规则不存在");

        keyAlarmRuleRepository.delete(keyAlarmRuleOptional.get());
        return Result.ok();
    }

    @Override
    public Result<String> modify(KeyAlarmRuleModifyParam keyAlarmRuleModifyParam) {
        Optional<KeyAlarmRule> keyAlarmRuleOptional = keyAlarmRuleRepository.findById(keyAlarmRuleModifyParam.getId());

        if (keyAlarmRuleOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "报警规则不存在");

        HashSet<User> users = new HashSet<>();
        String[] strings = keyAlarmRuleModifyParam.getUserIds().split(",");
        Long[] ids = Convert.toLongArray(strings);
        if (ids.length < 3) {
            return Result.no(Status.BAD_REQUEST, "报警接收人至少3个");
        }
        for (Long id : ids) {
            users.add(User.builder().id(id).build());
        }

        KeyAlarmRule keyAlarmRule = keyAlarmRuleOptional.get();
        keyAlarmRule.setDepartment(Department.builder().id(keyAlarmRuleModifyParam.getDeptId()).build());
        keyAlarmRule.setUsers(users);
        keyAlarmRule.setTypes(keyAlarmRuleModifyParam.getTypes());

        keyAlarmRuleRepository.save(keyAlarmRule);
        return Result.ok();
    }

    @Override
    public void export(KeyAlarmRuleExportParam keyAlarmRuleExportParam, HttpServletResponse response) {
        Specification<KeyAlarmRule> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(keyAlarmRuleExportParam.getDeptIds())) {
                String[] ids = keyAlarmRuleExportParam.getDeptIds().split(",");
                CriteriaBuilder.In<Object> in = cb.in(
                    root.get(KeyAlarmRule.Fields.department).get(BasicEntity.Fields.id)
                );
                for (String id : ids) in.value(Convert.toLong(id));
                predicates.add(in);
            }

            if (StrUtil.isNotEmpty(keyAlarmRuleExportParam.getTypes())) {
                String[] types = keyAlarmRuleExportParam.getTypes().split(",");
                for (String type : types) {
                    predicates.add(cb.like(root.get(KeyAlarmRule.Fields.types), "%" + type + "%"));
                }
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        List<KeyAlarmRule> keyAlarmRules = keyAlarmRuleRepository.findAll(specification);
        List<KeyAlarmRuleExportExcel> keyAlarmRuleExportExcels = keyAlarmRules
            .stream()
            .map(KeyAlarmRuleExportExcel::transform)
            .collect(Collectors.toList());

        try (
            DefaultStreamExcelBuilder<KeyAlarmRuleExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(KeyAlarmRuleExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(keyAlarmRuleExportExcels);

            String fileName = URLEncoder.encode("按键报警规则信息.xlsx", StandardCharsets.UTF_8);

            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);

            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出按键报警规则信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<PageVo<KeyAlarmRuleVo>> page(KeyAlarmRulePageParam keyAlarmRulePageParam, PageParam pageParam) {
        Specification<KeyAlarmRule> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(keyAlarmRulePageParam.getDeptIds())) {
                String[] ids = keyAlarmRulePageParam.getDeptIds().split(",");
                CriteriaBuilder.In<Object> in = cb.in(
                    root.get(KeyAlarmRule.Fields.department).get(BasicEntity.Fields.id)
                );
                for (String id : ids) in.value(Convert.toLong(id));
                predicates.add(in);
            }

            if (StrUtil.isNotEmpty(keyAlarmRulePageParam.getTypes())) {
                String[] types = keyAlarmRulePageParam.getTypes().split(",");
                for (String type : types) {
                    predicates.add(cb.like(root.get(KeyAlarmRule.Fields.types), "%" + type + "%"));
                }
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<KeyAlarmRule> pages = keyAlarmRuleRepository.findAll(specification, pageable);
        List<KeyAlarmRuleVo> keyAlarmRuleVos = pages
            .stream()
            .map(KeyAlarmRuleVo::transform)
            .collect(Collectors.toList());
        PageVo<KeyAlarmRuleVo> pageVo = PageVo.build(keyAlarmRuleVos, pages.getTotalElements());
        return Result.ok(pageVo);
    }
}
