/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.risk.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.yiring.app.domain.risk.*;
import com.yiring.app.domain.workArea.WorkArea;
import com.yiring.app.domain.workArea.WorkAreaRepository;
import com.yiring.app.param.risk.RiskPushConditionParam;
import com.yiring.app.param.risk.RiskPushSaveParam;
import com.yiring.app.service.risk.RiskPushService;
import com.yiring.app.vo.risk.RiskPushVo;
import com.yiring.app.vo.workArea.WorkAreaVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 风险推送
 *
 * @author LJ-2204
 * @date 2022/5/23
 */

@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class RiskPushServiceImpl implements RiskPushService {

    @Resource
    RiskPushRepository riskPushRepository;

    @Resource
    RiskHandlePushRepository riskHandlePushRepository;

    @Resource
    RiskWithdrawalPushRepository riskWithdrawalPushRepository;

    @Resource
    WorkAreaRepository workAreaRepository;

    @Override
    public Result<String> add(RiskPushSaveParam param) {
        Optional<WorkArea> workAreaOptional = workAreaRepository.findById(param.getWorkId());
        if (workAreaOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "工区不存在");
        }

        WorkArea workArea = workAreaOptional.get();

        RiskPush riskPush = RiskPush.builder().workArea(workArea).level(param.getLevel()).build();
        riskPushRepository.save(riskPush);

        return Result.ok();
    }

    @Override
    public Result<String> delete(IdParam idParam) {
        Optional<RiskPush> riskPushOptional = riskPushRepository.findById(idParam.getId());

        if (riskPushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "规则不存在");
        }

        riskPushRepository.delete(riskPushOptional.get());
        return Result.ok();
    }

    @Override
    public Result<PageVo<RiskPushVo>> page(RiskPushConditionParam param, PageParam pageParam) {
        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<RiskPush> riskPushes = riskPushRepository.findAll(condition(param), pageable);
        List<RiskPushVo> riskPushVos = riskPushes
            .stream()
            .map(riskPush -> {
                RiskPushVo riskPushVo = RiskPushVo.transform(riskPush);
                if (ObjectUtil.isNotEmpty(riskPush.getWorkArea())) {
                    WorkArea workArea = riskPush.getWorkArea();
                    WorkAreaVo workAreaVo = Convert.convert(WorkAreaVo.class, workArea);
                    riskPushVo.setWorkAreaVo(workAreaVo);
                }

                // 查询处置推送总数
                List<RiskHandlePush> riskHandlePushes = riskHandlePushRepository.findAll(
                    Example.of(RiskHandlePush.builder().riskPush(riskPush).build())
                );
                riskPushVo.setHandlePush(Convert.toStr(riskHandlePushes.size()));

                // 查询撤离推送总数
                List<RiskWithdrawalPush> riskWithdrawalPushes = riskWithdrawalPushRepository.findAll(
                    Example.of(RiskWithdrawalPush.builder().riskPush(riskPush).build())
                );
                riskPushVo.setWithdrawalPush(Convert.toStr(riskWithdrawalPushes.size()));

                return riskPushVo;
            })
            .collect(Collectors.toList());
        PageVo<RiskPushVo> pageVo = PageVo.build(riskPushVos, riskPushes.getTotalElements());
        return Result.ok(pageVo);
    }

    @Override
    public Result<RiskPushVo> info(IdParam idParam) {
        Optional<RiskPush> riskPushOptional = riskPushRepository.findById(idParam.getId());

        if (riskPushOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "风险规则不存在");
        }

        RiskPush riskPush = riskPushOptional.get();
        RiskPushVo riskPushVo = Convert.convert(RiskPushVo.class, riskPush);
        if (ObjectUtil.isNotEmpty(riskPush.getWorkArea())) {
            WorkArea workArea = riskPush.getWorkArea();
            WorkAreaVo workAreaVo = Convert.convert(WorkAreaVo.class, workArea);
            riskPushVo.setWorkAreaVo(workAreaVo);
        }

        return Result.ok(riskPushVo);
    }

    public Specification<RiskPush> condition(RiskPushConditionParam param) {
        return (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (ObjectUtil.isNotEmpty(param.getWorkId())) {
                predicates.add(
                    cb.equal(root.get(RiskPush.Fields.workArea).get(BasicEntity.Fields.id), param.getWorkId())
                );
            }

            if (ObjectUtil.isNotEmpty(param.getLevel())) {
                predicates.add(cb.equal(root.get(RiskPush.Fields.level), param.getLevel()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));
            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };
    }
}
